@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, KATO Noriaki
+ * Copyright (c) 2006-2009, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,8 +27,8 @@
  * \brief アルゴリズム
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2007-06-20
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.5.0/cstl/algorithm.h $
- * $Id: algorithm.h 190 2008-01-03 07:56:11Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.0/cstl/algorithm.h $
+ * $Id: algorithm.h 264 2009-04-17 02:29:36Z katono $
  */
 #ifndef CSTL_ALGORITHM_H_INCLUDED
 #define CSTL_ALGORITHM_H_INCLUDED
@@ -63,6 +63,7 @@ void Name##_push_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *
 void Name##_pop_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *));\
 void Name##_make_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *));\
 void Name##_sort_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *));\
+void Name##_partial_sort(Name *self, size_t idx, size_t sort_n, size_t n, int (*comp)(const void *, const void *));\
 
 
 #define CSTL_ALGORITHM_IMPLEMENT(Name, Type, DIRECT_ACCESS)	\
@@ -116,7 +117,8 @@ void Name##_sort(Name *self, size_t idx, size_t n, int (*comp)(const void *, con
 			if (comp(&DIRECT_ACCESS(self, middle), &DIRECT_ACCESS(self, r)) > 0) {\
 				CSTL_ALGORITHM_SWAP(middle, r, tmp, DIRECT_ACCESS);\
 			}\
-			CSTL_ALGORITHM_SWAP(middle, r - 1, tmp, DIRECT_ACCESS); /* r - 1の要素をを枢軸にする */\
+			/* r - 1の要素をを枢軸にする */\
+			CSTL_ALGORITHM_SWAP(middle, r - 1, tmp, DIRECT_ACCESS);\
 			/* partition */\
 			i = l;\
 			j = r - 1;\
@@ -430,7 +432,7 @@ int Name##_merge(Name *self, size_t idx, \
 	assert(Name##_size(y) >= yn && "merge");\
 	assert(Name##_size(y) > yidx && "merge");\
 	assert(comp && "merge");\
-	if (!Name##_insert_n_no_elem(self, idx, xn + yn)) {\
+	if (!Name##_insert_n_no_data(self, idx, xn + yn)) {\
 		return 0;\
 	}\
 	i = j = k = 0;\
@@ -573,6 +575,28 @@ void Name##_sort_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *
 	}\
 }\
 \
+void Name##_partial_sort(Name *self, size_t idx, size_t sort_n, size_t n, int (*comp)(const void *, const void *))\
+{\
+	register size_t i;\
+	Type tmp;\
+	assert(self && "partial_sort");\
+	assert(self->magic == self && "partial_sort");\
+	assert(Name##_size(self) >= idx + n && "partial_sort");\
+	assert(Name##_size(self) >= n && "partial_sort");\
+	assert(Name##_size(self) > idx && "partial_sort");\
+	assert(sort_n <= n && "partial_sort");\
+	assert(comp && "partial_sort");\
+	Name##_make_heap(self, idx, sort_n, comp);\
+	for (i = idx + sort_n; i < idx + n; i++) {\
+		if (comp(&DIRECT_ACCESS(self, idx), &DIRECT_ACCESS(self, i)) > 0) {\
+			Name##_pop_heap(self, idx, sort_n, comp);\
+			CSTL_ALGORITHM_SWAP(idx + sort_n - 1, i, tmp, DIRECT_ACCESS);\
+			Name##_push_heap(self, idx, sort_n, comp);\
+		}\
+	}\
+	Name##_sort_heap(self, idx, sort_n, comp);\
+}\
+\
 
 
 #endif /* CSTL_ALGORITHM_H_INCLUDED */
