@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006, KATO Noriaki
+ * Copyright (c) 2006-2009, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,13 +27,12 @@
  * \brief listコンテナ
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-02-26
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.4.0/cstl/list.h $
- * $Id: list.h 201 2008-02-16 06:26:56Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.0/cstl/list.h $
+ * $Id: list.h 264 2009-04-17 02:29:36Z katono $
  */
 #ifndef CSTL_LIST_H_INCLUDED
 #define CSTL_LIST_H_INCLUDED
 
-#include <stddef.h>
 #include <stdlib.h>
 #include <assert.h>
 
@@ -56,9 +55,6 @@
 #define CSTL_LIST_END(self)		(self)
 #define CSTL_LIST_RBEGIN(self)	(self)->prev
 #define CSTL_LIST_REND(self)	(self)
-#define CSTL_LIST_NEXT(pos)		(pos)->next
-#define CSTL_LIST_PREV(pos)		(pos)->prev
-#define CSTL_LIST_AT(pos)		(pos)->elem
 
 
 /*! 
@@ -68,38 +64,38 @@
  * \param Type 要素の型
  */
 #define CSTL_LIST_INTERFACE(Name, Type)	\
-typedef struct Name##_t Name;\
-/*! 
- * \brief イテレータ
+typedef struct Name Name;\
+/*! \
+ * \brief イテレータ\
  */\
-typedef struct Name##_t *Name##Iterator;\
+typedef struct Name *Name##Iterator;\
 \
 CSTL_LIST_BEGIN_EXTERN_C()\
 Name *Name##_new(void);\
 void Name##_delete(Name *self);\
-int Name##_push_back(Name *self, Type elem);\
-int Name##_push_front(Name *self, Type elem);\
-Type Name##_pop_front(Name *self);\
-Type Name##_pop_back(Name *self);\
+int Name##_push_back(Name *self, Type data);\
+int Name##_push_front(Name *self, Type data);\
+void Name##_pop_front(Name *self);\
+void Name##_pop_back(Name *self);\
 int Name##_empty(Name *self);\
 size_t Name##_size(Name *self);\
 void Name##_clear(Name *self);\
-Type *Name##_at(Name##Iterator pos);\
-Type Name##_front(Name *self);\
-Type Name##_back(Name *self);\
+Type *Name##_data(Name##Iterator pos);\
+Type *Name##_front(Name *self);\
+Type *Name##_back(Name *self);\
 Name##Iterator Name##_begin(Name *self);\
 Name##Iterator Name##_end(Name *self);\
 Name##Iterator Name##_rbegin(Name *self);\
 Name##Iterator Name##_rend(Name *self);\
 Name##Iterator Name##_next(Name##Iterator pos);\
 Name##Iterator Name##_prev(Name##Iterator pos);\
-Name##Iterator Name##_insert(Name *self, Name##Iterator pos, Type elem);\
-int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type elem);\
-int Name##_insert_array(Name *self, Name##Iterator pos, const Type *elems, size_t n);\
+Name##Iterator Name##_insert(Name *self, Name##Iterator pos, Type data);\
+int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type data);\
+int Name##_insert_array(Name *self, Name##Iterator pos, Type const *data, size_t n);\
 int Name##_insert_range(Name *self, Name##Iterator pos, Name##Iterator first, Name##Iterator last);\
 Name##Iterator Name##_erase(Name *self, Name##Iterator pos);\
 Name##Iterator Name##_erase_range(Name *self, Name##Iterator first, Name##Iterator last);\
-int Name##_resize(Name *self, size_t n, Type elem);\
+int Name##_resize(Name *self, size_t n, Type data);\
 void Name##_swap(Name *self, Name *x);\
 void Name##_splice(Name *self, Name##Iterator pos, Name *x, Name##Iterator first, Name##Iterator last);\
 void Name##_sort(Name *self, int (*comp)(const void *, const void *));\
@@ -115,15 +111,14 @@ CSTL_LIST_END_EXTERN_C()\
  * \param Type 要素の型
  */
 #define CSTL_LIST_IMPLEMENT(Name, Type)	\
-typedef struct Name##_t Name##Node;\
-/*! 
- * \brief list構造体
+/*! \
+ * \brief list構造体\
  */\
-struct Name##_t {\
-	Name##Node *prev;\
-	Name##Node *next;\
-	Type elem;\
-	CSTL_LIST_MAGIC(Name##Node *magic;)\
+struct Name {\
+	Name *prev;\
+	Name *next;\
+	Type data;\
+	CSTL_LIST_MAGIC(Name *magic;)\
 };\
 \
 Name *Name##_new(void)\
@@ -139,69 +134,55 @@ Name *Name##_new(void)\
 \
 void Name##_delete(Name *self)\
 {\
-	assert(self && "List_delete");\
+	if (!self) return;\
 	assert(self->magic == self && "List_delete");\
 	Name##_clear(self);\
 	CSTL_LIST_MAGIC(self->magic = 0);\
 	free(self);\
 }\
 \
-int Name##_push_back(Name *self, Type elem)\
+static Name##Iterator Name##_insert_ref(Name *self, Name##Iterator pos, Type const *data)\
+{\
+	Name *node;\
+	node = (Name *) malloc(sizeof(Name));\
+	if (!node) return 0;\
+	node->data = *data;\
+	node->next = pos;\
+	node->prev = pos->prev;\
+	pos->prev = node;\
+	node->prev->next = node;\
+	CSTL_LIST_MAGIC(node->magic = self);\
+	return node;\
+}\
+\
+int Name##_push_back(Name *self, Type data)\
 {\
-	Name##Node *node;\
-	Name##Node *pos;\
 	assert(self && "List_push_back");\
 	assert(self->magic == self && "List_push_back");\
-	node = (Name##Node *) malloc(sizeof(Name##Node));\
-	if (!node) return 0;\
-	pos = CSTL_LIST_END(self);\
-	node->elem = elem;\
-	node->next = pos;\
-	node->prev = pos->prev;\
-	pos->prev = node;\
-	node->prev->next = node;\
-	CSTL_LIST_MAGIC(node->magic = self);\
-	return 1;\
+	return Name##_insert_ref(self, CSTL_LIST_END(self), &data) ? 1 : 0;\
 }\
 \
-int Name##_push_front(Name *self, Type elem)\
+int Name##_push_front(Name *self, Type data)\
 {\
-	Name##Node *node;\
-	Name##Node *pos;\
 	assert(self && "List_push_front");\
 	assert(self->magic == self && "List_push_front");\
-	node = (Name##Node *) malloc(sizeof(Name##Node));\
-	if (!node) return 0;\
-	pos = CSTL_LIST_BEGIN(self);\
-	node->elem = elem;\
-	node->next = pos;\
-	node->prev = pos->prev;\
-	pos->prev = node;\
-	node->prev->next = node;\
-	CSTL_LIST_MAGIC(node->magic = self);\
-	return 1;\
+	return Name##_insert_ref(self, CSTL_LIST_BEGIN(self), &data) ? 1 : 0;\
 }\
 \
-Type Name##_pop_front(Name *self)\
+void Name##_pop_front(Name *self)\
 {\
-	Type elem;\
 	assert(self && "List_pop_front");\
 	assert(self->magic == self && "List_pop_front");\
 	assert(!Name##_empty(self) && "List_pop_front");\
-	elem = CSTL_LIST_AT(CSTL_LIST_BEGIN(self));\
 	Name##_erase(self, CSTL_LIST_BEGIN(self));\
-	return elem;\
 }\
 \
-Type Name##_pop_back(Name *self)\
+void Name##_pop_back(Name *self)\
 {\
-	Type elem;\
 	assert(self && "List_pop_back");\
 	assert(self->magic == self && "List_pop_back");\
 	assert(!Name##_empty(self) && "List_pop_back");\
-	elem = CSTL_LIST_AT(CSTL_LIST_RBEGIN(self));\
 	Name##_erase(self, CSTL_LIST_RBEGIN(self));\
-	return elem;\
 }\
 \
 int Name##_empty(Name *self)\
@@ -217,7 +198,7 @@ size_t Name##_size(Name *self)\
 	register size_t count = 0;\
 	assert(self && "List_size");\
 	assert(self->magic == self && "List_size");\
-	for (pos = CSTL_LIST_BEGIN(self); pos != CSTL_LIST_END(self); pos = CSTL_LIST_NEXT(pos)) {\
+	for (pos = CSTL_LIST_BEGIN(self); pos != CSTL_LIST_END(self); pos = pos->next) {\
 		count++;\
 	}\
 	return count;\
@@ -230,28 +211,28 @@ void Name##_clear(Name *self)\
 	Name##_erase_range(self, CSTL_LIST_BEGIN(self), CSTL_LIST_END(self));\
 }\
 \
-Type *Name##_at(Name##Iterator pos)\
+Type *Name##_data(Name##Iterator pos)\
 {\
-	assert(pos && "List_at");\
-	assert(pos->magic && "List_at");\
-	assert(pos->magic != pos && "List_at");\
-	return &CSTL_LIST_AT(pos);\
+	assert(pos && "List_data");\
+	assert(pos->magic && "List_data");\
+	assert(pos->magic != pos && "List_data");\
+	return &pos->data;\
 }\
 \
-Type Name##_front(Name *self)\
+Type *Name##_front(Name *self)\
 {\
 	assert(self && "List_front");\
 	assert(self->magic == self && "List_front");\
 	assert(!Name##_empty(self) && "List_front");\
-	return self->next->elem;\
+	return &CSTL_LIST_BEGIN(self)->data;\
 }\
 \
-Type Name##_back(Name *self)\
+Type *Name##_back(Name *self)\
 {\
 	assert(self && "List_back");\
 	assert(self->magic == self && "List_back");\
 	assert(!Name##_empty(self) && "List_back");\
-	return self->prev->elem;\
+	return &CSTL_LIST_RBEGIN(self)->data;\
 }\
 \
 Name##Iterator Name##_begin(Name *self)\
@@ -287,7 +268,7 @@ Name##Iterator Name##_next(Name##Iterator pos)\
 	assert(pos && "List_next");\
 	assert(pos->magic && "List_next");\
 	assert(pos->magic != pos && "List_next");\
-	return CSTL_LIST_NEXT(pos);\
+	return pos->next;\
 }\
 \
 Name##Iterator Name##_prev(Name##Iterator pos)\
@@ -295,28 +276,19 @@ Name##Iterator Name##_prev(Name##Iterator pos)\
 	assert(pos && "List_prev");\
 	assert(pos->magic && "List_prev");\
 	assert(pos->magic != pos && "List_prev");\
-	return CSTL_LIST_PREV(pos);\
+	return pos->prev;\
 }\
 \
-Name##Iterator Name##_insert(Name *self, Name##Iterator pos, Type elem)\
+Name##Iterator Name##_insert(Name *self, Name##Iterator pos, Type data)\
 {\
-	Name##Node *node;\
 	assert(self && "List_insert");\
 	assert(self->magic == self && "List_insert");\
 	assert(pos && "List_insert");\
 	assert(pos->magic == self && "List_insert");\
-	node = (Name##Node *) malloc(sizeof(Name##Node));\
-	if (!node) return 0;\
-	node->elem = elem;\
-	node->next = pos;\
-	node->prev = pos->prev;\
-	pos->prev = node;\
-	node->prev->next = node;\
-	CSTL_LIST_MAGIC(node->magic = self);\
-	return node;\
+	return Name##_insert_ref(self, pos, &data);\
 }\
 \
-int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type elem)\
+int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type data)\
 {\
 	Name x;\
 	register size_t i;\
@@ -328,7 +300,7 @@ int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type elem)\
 	x.prev = &x;\
 	CSTL_LIST_MAGIC(x.magic = &x);\
 	for (i = 0; i < n; i++) {\
-		if (!Name##_push_back(&x, elem)) {\
+		if (!Name##_insert_ref(&x, CSTL_LIST_END(&x), &data)) {\
 			Name##_clear(&x);\
 			return 0;\
 		}\
@@ -337,7 +309,7 @@ int Name##_insert_n(Name *self, Name##Iterator pos, size_t n, Type elem)\
 	return 1;\
 }\
 \
-int Name##_insert_array(Name *self, Name##Iterator pos, const Type *elems, size_t n)\
+int Name##_insert_array(Name *self, Name##Iterator pos, Type const *data, size_t n)\
 {\
 	Name x;\
 	register size_t i;\
@@ -345,12 +317,12 @@ int Name##_insert_array(Name *self, Name##Iterator pos, const Type *elems, size_
 	assert(self->magic == self && "List_insert_array");\
 	assert(pos && "List_insert_array");\
 	assert(pos->magic == self && "List_insert_array");\
-	assert(elems && "List_insert_array");\
+	assert(data && "List_insert_array");\
 	x.next = &x;\
 	x.prev = &x;\
 	CSTL_LIST_MAGIC(x.magic = &x);\
 	for (i = 0; i < n; i++) {\
-		if (!Name##_push_back(&x, ((Type *) elems)[i])) {\
+		if (!Name##_insert_ref(&x, CSTL_LIST_END(&x), &data[i])) {\
 			Name##_clear(&x);\
 			return 0;\
 		}\
@@ -374,9 +346,9 @@ int Name##_insert_range(Name *self, Name##Iterator pos, Name##Iterator first, Na
 	x.next = &x;\
 	x.prev = &x;\
 	CSTL_LIST_MAGIC(x.magic = &x);\
-	for (i = first; i != last; i = CSTL_LIST_NEXT(i)) {\
+	for (i = first; i != last; i = i->next) {\
 		assert(i->magic && "List_insert_range");\
-		if (!Name##_push_back(&x, CSTL_LIST_AT(i))) {\
+		if (!Name##_insert_ref(&x, CSTL_LIST_END(&x), &i->data)) {\
 			Name##_clear(&x);\
 			return 0;\
 		}\
@@ -387,7 +359,7 @@ int Name##_insert_range(Name *self, Name##Iterator pos, Name##Iterator first, Na
 \
 Name##Iterator Name##_erase(Name *self, Name##Iterator pos)\
 {\
-	Name##Node *node;\
+	Name *node;\
 	assert(self && "List_erase");\
 	assert(self->magic == self && "List_erase");\
 	assert(pos && "List_erase");\
@@ -419,7 +391,7 @@ Name##Iterator Name##_erase_range(Name *self, Name##Iterator first, Name##Iterat
 	return pos;\
 }\
 \
-int Name##_resize(Name *self, size_t n, Type elem)\
+int Name##_resize(Name *self, size_t n, Type data)\
 {\
 	register size_t i;\
 	size_t size;\
@@ -436,7 +408,7 @@ int Name##_resize(Name *self, size_t n, Type elem)\
 		x.prev = &x;\
 		CSTL_LIST_MAGIC(x.magic = &x);\
 		for (i = 0; i < n - size; i++) {\
-			if (!Name##_push_back(&x, elem)) {\
+			if (!Name##_insert_ref(&x, CSTL_LIST_END(&x), &data)) {\
 				Name##_clear(&x);\
 				return 0;\
 			}\
@@ -448,19 +420,19 @@ int Name##_resize(Name *self, size_t n, Type elem)\
 \
 void Name##_swap(Name *self, Name *x)\
 {\
-	Name##Node *tmp_next;\
-	Name##Node *tmp_prev;\
+	Name *tmp_next;\
+	Name *tmp_prev;\
 	CSTL_LIST_MAGIC(register Name##Iterator pos);\
 	assert(self && "List_swap");\
 	assert(x && "List_swap");\
 	assert(self->magic == self && "List_swap");\
 	assert(x->magic == x && "List_swap");\
 	CSTL_LIST_MAGIC(\
-		for (pos = CSTL_LIST_BEGIN(self); pos != CSTL_LIST_END(self); pos = CSTL_LIST_NEXT(pos)) {\
+		for (pos = CSTL_LIST_BEGIN(self); pos != CSTL_LIST_END(self); pos = pos->next) {\
 			assert(pos->magic == self && "List_swap");\
 			pos->magic = x;\
 		}\
-		for (pos = CSTL_LIST_BEGIN(x); pos != CSTL_LIST_END(x); pos = CSTL_LIST_NEXT(pos)) {\
+		for (pos = CSTL_LIST_BEGIN(x); pos != CSTL_LIST_END(x); pos = pos->next) {\
 			assert(pos->magic == x && "List_swap");\
 			pos->magic = self;\
 		}\
@@ -479,7 +451,7 @@ void Name##_swap(Name *self, Name *x)\
 \
 void Name##_splice(Name *self, Name##Iterator pos, Name *x, Name##Iterator first, Name##Iterator last)\
 {\
-	Name##Node *tmp;\
+	Name *tmp;\
 	CSTL_LIST_MAGIC(register Name##Iterator i);\
 	assert(self && "List_splice");\
 	assert(self->magic == self && "List_splice");\
@@ -493,7 +465,7 @@ void Name##_splice(Name *self, Name##Iterator pos, Name *x, Name##Iterator first
 	assert(last->magic == x && "List_splice");\
 	if (first == last || pos == last) return;\
 	CSTL_LIST_MAGIC(\
-		for (i = first; i != last; i = CSTL_LIST_NEXT(i)) {\
+		for (i = first; i != last; i = i->next) {\
 			assert(i != pos && "List_splice");\
 			assert(i->magic == x && "List_splice");\
 			i->magic = self;\
@@ -508,19 +480,19 @@ void Name##_splice(Name *self, Name##Iterator pos, Name *x, Name##Iterator first
 	last->prev = tmp;\
 }\
 \
-static Name##Node *Name##_merge_list(Name##Node *x, Name##Node *y, int (*comp)(const void *, const void *))\
+static Name *Name##_merge_list(Name *x, Name *y, int (*comp)(const void *, const void *))\
 {\
-	register Name##Node *p;\
-	Name##Node *xlast;\
-	Name##Node *ylast;\
-	Name##Node head;\
+	register Name *p;\
+	Name *xlast;\
+	Name *ylast;\
+	Name head;\
 	xlast = x->prev;\
 	ylast = y->prev;\
 	p = &head;\
 	x->prev->next = &head;\
 	y->prev->next = &head;\
 	while (x != &head && y != &head) {\
-		if (comp(&CSTL_LIST_AT(x), &CSTL_LIST_AT(y)) <= 0) {\
+		if (comp(&x->data, &y->data) <= 0) {\
 			p->next = x;\
 			x->prev = p;\
 			p = x;\
@@ -546,11 +518,11 @@ static Name##Node *Name##_merge_list(Name##Node *x, Name##Node *y, int (*comp)(c
 	return head.next;\
 }\
 \
-static Name##Node *Name##_merge_sort(Name##Node *first, int (*comp)(const void *, const void *))\
+static Name *Name##_merge_sort(Name *first, int (*comp)(const void *, const void *))\
 {\
-	register Name##Node *x;\
-	register Name##Node *y;\
-	Name##Node *tmp;\
+	register Name *x;\
+	register Name *y;\
+	Name *tmp;\
 	if (first->next == first) {\
 		return first;\
 	}\
@@ -573,8 +545,8 @@ static Name##Node *Name##_merge_sort(Name##Node *first, int (*comp)(const void *
 \
 void Name##_sort(Name *self, int (*comp)(const void *, const void *))\
 {\
-	Name##Node *first;\
-	Name##Node *last;\
+	Name *first;\
+	Name *last;\
 	assert(self && "List_sort");\
 	assert(self->magic == self && "List_sort");\
 	assert(comp && "List_sort");\
@@ -596,10 +568,10 @@ void Name##_sort(Name *self, int (*comp)(const void *, const void *))\
 \
 void Name##_merge(Name *self, Name *x, int (*comp)(const void *, const void *))\
 {\
-	Name##Node *first1;\
-	Name##Node *last1;\
-	Name##Node *first2;\
-	Name##Node *last2;\
+	Name *first1;\
+	Name *last1;\
+	Name *first2;\
+	Name *last2;\
 	CSTL_LIST_MAGIC(register Name##Iterator pos);\
 	assert(self && "List_merge");\
 	assert(self->magic == self && "List_merge");\
@@ -612,7 +584,7 @@ void Name##_merge(Name *self, Name *x, int (*comp)(const void *, const void *))\
 		return;\
 	}\
 	CSTL_LIST_MAGIC(\
-		for (pos = CSTL_LIST_BEGIN(x); pos != CSTL_LIST_END(x); pos = CSTL_LIST_NEXT(pos)) {\
+		for (pos = CSTL_LIST_BEGIN(x); pos != CSTL_LIST_END(x); pos = pos->next) {\
 			assert(pos->magic == x && "List_merge");\
 			pos->magic = self;\
 		}\
@@ -638,8 +610,8 @@ void Name##_merge(Name *self, Name *x, int (*comp)(const void *, const void *))\
 \
 void Name##_reverse(Name *self)\
 {\
-	register Name##Node *p;\
-	register Name##Node *tmp;\
+	register Name *p;\
+	register Name *tmp;\
 	assert(self && "List_reverse");\
 	assert(self->magic == self && "List_reverse");\
 	p = self;\
