@@ -24,18 +24,20 @@ int main(void)
 		printf("%02x\n", *UCharRing_at(x, i));
 	}
 	while (!UCharRing_empty(x)) {
-		printf("%02x\n", UCharRing_pop_front(x));
+		printf("%02x\n", *UCharRing_front(x));
+		UCharRing_pop_front(x);
 	}
 	UCharRing_delete(x);
 
-	/* ĂяoŗpӂIuWFNgƃobt@𗘗pꍇ */
+	/* 呼び出し側で用意したオブジェクトとバッファを利用する場合 */
 	UCharRing_init(&y, buf, NELEMS(buf));
 	UCharRing_push_back(&y, 0x01);
 	UCharRing_push_back(&y, 0x02);
 	while (!UCharRing_empty(&y)) {
-		printf("%02x\n", UCharRing_pop_front(&y));
+		printf("%02x\n", *UCharRing_front(&y));
+		UCharRing_pop_front(&y);
 	}
-	/* UCharRing_delete(&y)ĂяoĂ͂Ȃ */
+	/* UCharRing_delete(&y)を呼び出してはいけない */
 	return 0;
 }
 
